@@ -37,7 +37,7 @@ write_err_and_exit (gint fd,
   
   if (debug)
     {
-      debugstring = g_string_new ("");
+      debugstring = g_string_new (NULL);
       g_string_append (debugstring,
 		       g_strdup_printf ("writing error code %d and errno %d",
 					msg, en));
@@ -76,13 +76,14 @@ WinMain (struct HINSTANCE__ *hInstance,
   int handle;
   int no_error = CHILD_NO_ERROR;
   int zero = 0;
+  gint file_and_argv_zero = 0;
   gchar **new_argv;
 
   SETUP_DEBUG();
 
   if (debug)
     {
-      debugstring = g_string_new ("");
+      debugstring = g_string_new (NULL);
 
       g_string_append (debugstring,
 		       g_strdup_printf ("g-spawn-win32-helper: "
@@ -105,6 +106,10 @@ WinMain (struct HINSTANCE__ *hInstance,
    */
   child_err_report_fd = atoi (__argv[ARG_CHILD_ERR_REPORT]);
 
+  /* Hack to implement G_SPAWN_FILE_AND_ARGV_ZERO */
+  if (__argv[ARG_CHILD_ERR_REPORT][strlen (__argv[ARG_CHILD_ERR_REPORT]) - 1] == '#')
+    file_and_argv_zero = 1;
+
   /* argv[ARG_STDIN..ARG_STDERR] are the file descriptors that should
    * be dup2'd to stdin, stdout and stderr, '-' if the corresponding
    * std* should be let alone, and 'z' if it should be connected to
@@ -212,7 +217,7 @@ WinMain (struct HINSTANCE__ *hInstance,
 
   if (debug)
     {
-      debugstring = g_string_new ("");
+      debugstring = g_string_new (NULL);
       g_string_append (debugstring,
 		       g_strdup_printf ("calling %s %s mode=%s argv: ",
 					(__argv[ARG_USE_PATH][0] == 'y' ?
@@ -220,7 +225,7 @@ WinMain (struct HINSTANCE__ *hInstance,
 					__argv[ARG_PROGRAM],
 					(mode == P_WAIT ?
 					 "P_WAIT" : "P_NOWAIT")));
-      i = ARG_PROGRAM+1;
+      i = ARG_PROGRAM + 1 + file_and_argv_zero;
       while (new_argv[i])
 	{
 	  g_string_append (debugstring, new_argv[i++]);
@@ -231,13 +236,13 @@ WinMain (struct HINSTANCE__ *hInstance,
     }
 
   if (new_argv[ARG_USE_PATH][0] == 'y')
-    handle = spawnvp (mode, __argv[ARG_PROGRAM], new_argv+ARG_PROGRAM);
+    handle = spawnvp (mode, __argv[ARG_PROGRAM], new_argv + ARG_PROGRAM + file_and_argv_zero);
   else
-    handle = spawnv (mode, __argv[ARG_PROGRAM], new_argv+ARG_PROGRAM);
+    handle = spawnv (mode, __argv[ARG_PROGRAM], new_argv + ARG_PROGRAM + file_and_argv_zero);
 
   if (debug)
     {
-      debugstring = g_string_new ("");
+      debugstring = g_string_new (NULL);
       g_string_append (debugstring,
 		       g_strdup_printf ("%s returned %#x",
 					(__argv[ARG_USE_PATH][0] == 'y' ?
